==> drm-1.0.tar.gz: drm-1.0/common/DrmReceiver.h <==
/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	See DrmReceiver.cpp
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#if !defined(DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_)
#define DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_

#include <iostream>
#include "Parameter.h"
#include "Buffer.h"
#include "Data.h"
#include "OFDM.h"
#include "DRMSignalIO.h"
#include "MSCMultiplexer.h"
#include "InputResample.h"
#include "datadecoding/DataDecoder.h"
#include "sourcedecoders/AudioSourceDecoder.h"
#include "mlc/MLC.h"
#include "interleaver/SymbolInterleaver.h"
#include "ofdmcellmapping/OFDMCellMapping.h"
#include "chanest/ChannelEstimation.h"
#include "sync/FreqSyncAcq.h"
#include "sync/TimeSync.h"
#include "sync/SyncUsingPil.h"
#include "AMDemodulation.h"
#ifdef _WIN32
# include "../../Windows/source/sound.h"
#else
# include "source/sound.h"
#endif


/* Definitions ****************************************************************/
/* Number of FAC frames until the acquisition is activated in case a signal
   was successfully decoded */
#define	NUM_FAC_FRA_U_ACQ_WITH			12

/* Number of FAC frames until the acquisition is activated in case no signal
   could be decoded after previous acquisition try */
#define	NUM_FAC_FRA_U_ACQ_WITHOUT		9

/* Number of FAC blocks for delayed tracking mode switch (caused by time needed
   for initalizing the channel estimation */
#define NUM_FAC_DEL_TRACK_SWITCH		2


/* Classes ********************************************************************/
class CDRMReceiver
{
public:
	/* Acquisition state of receiver */
	enum EAcqStat {AS_NO_SIGNAL, AS_WITH_SIGNAL};

	/* Receiver state */
	enum ERecState {RS_TRACKING, RS_ACQUISITION};

	/* RM: Receiver mode (analog or digital demodulation) */
	enum ERecMode {RM_DRM, RM_AM, RM_NONE};


	CDRMReceiver() : eAcquiState(AS_NO_SIGNAL), iAcquDetecCnt(0),
		iGoodSignCnt(0), bWasFreqAcqu(TRUE), bDoInitRun(FALSE),
		eReceiverMode(RM_DRM), 	eNewReceiverMode(RM_NONE),
		ReceiveData(&SoundInterface), WriteData(&SoundInterface),
		rInitResampleOffset((_REAL) 0.0) {}
	virtual ~CDRMReceiver() {}

	/* For GUI */
	void					Init();
	void					Start();
	void					Stop();
	EAcqStat				GetReceiverState() {return eAcquiState;}
	ERecMode				GetReceiverMode() {return eReceiverMode;}
	void					SetReceiverMode(ERecMode eNewMode)
								{eNewReceiverMode = eNewMode;}

	void					SetInitResOff(_REAL rNRO)
								{rInitResampleOffset = rNRO;}

	/* Get pointer to internal modules */
	CUtilizeFACData*		GetFAC() {return &UtilizeFACData;}
	CUtilizeSDCData*		GetSDC() {return &UtilizeSDCData;}
	CTimeSync*				GetTimeSync() {return &TimeSync;}
	CChannelEstimation*		GetChanEst() {return &ChannelEstimation;}
	CFACMLCDecoder*			GetFACMLC() {return &FACMLCDecoder;}
	CSDCMLCDecoder*			GetSDCMLC() {return &SDCMLCDecoder;}
	CMSCMLCDecoder*			GetMSCMLC() {return &MSCMLCDecoder;}
	CReceiveData*			GetReceiver() {return &ReceiveData;}
	COFDMDemodulation*		GetOFDMDemod() {return &OFDMDemodulation;}
	CSyncUsingPil*			GetSyncUsPil() {return &SyncUsingPil;}
	CWriteData*				GetWriteData() {return &WriteData;}
	CSound*					GetSoundInterface() {return &SoundInterface;}
	CDataDecoder*			GetDataDecoder() {return &DataDecoder;}


	CParameter*				GetParameters() {return &ReceiverParam;}
	void					StartParameters(CParameter& Param);
	void					SetInStartMode();
	void					SetInTrackingMode();
	void					SetInTrackingModeDelayed();

	void					InitsForAllModules();

	void					InitsForWaveMode();
	void					InitsForSpectrumOccup();
	void					InitsForNoDecBitsSDC();
	void					InitsForAudParam();
	void					InitsForDataParam();
	void					InitsForInterlDepth();
	void					InitsForMSCCodSche();
	void					InitsForSDCCodSche();
	void					InitsForMSC();
	void					InitsForMSCDemux();


protected:
	void					Run();
	void					DetectAcqui();
	void					InitReceiverMode();

	/* Modules */
	CReceiveData			ReceiveData;
	CInputResample			InputResample;
	CFreqSyncAcq			FreqSyncAcq;
	CTimeSync				TimeSync;
	COFDMDemodulation		OFDMDemodulation;
	CSyncUsingPil			SyncUsingPil;
	CChannelEstimation		ChannelEstimation;
	COFDMCellDemapping		OFDMCellDemapping;
	CFACMLCDecoder			FACMLCDecoder;
	CUtilizeFACData			UtilizeFACData;
	CSDCMLCDecoder			SDCMLCDecoder;
	CUtilizeSDCData			UtilizeSDCData;
	CSymbDeinterleaver		SymbDeinterleaver;
	CMSCMLCDecoder			MSCMLCDecoder;
	CMSCDemultiplexer		MSCDemultiplexer;
	CAudioSourceDecoder		AudioSourceDecoder;
	CDataDecoder			DataDecoder;
	CWriteData				WriteData;
	CAMDemodulation			AMDemodulation;

	/* Parameters */
	CParameter				ReceiverParam;

	/* Buffers */
	CSingleBuffer<_REAL>	RecDataBuf;
	CCyclicBuffer<_REAL>	FreqSyncAcqBuf;
	CCyclicBuffer<_REAL>	InpResBuf;
	CSingleBuffer<_REAL>	TimeSyncBuf;
	CSingleBuffer<_COMPLEX>	OFDMDemodBuf;
	CSingleBuffer<_COMPLEX>	SyncUsingPilBuf;
	CSingleBuffer<CEquSig>	ChanEstBuf;
	CCyclicBuffer<CEquSig>	MSCCarDemapBuf;
	CCyclicBuffer<CEquSig>	FACCarDemapBuf;
	CCyclicBuffer<CEquSig>	SDCCarDemapBuf;
	CSingleBuffer<CEquSig>	DeintlBuf;
	CSingleBuffer<_BINARY>	FACDecBuf;
	CSingleBuffer<_BINARY>	SDCDecBuf;
	CSingleBuffer<_BINARY>	MSCMLCDecBuf;
	CSingleBuffer<_BINARY>	MSCDeMUXBufAud;
	CSingleBuffer<_BINARY>	MSCDeMUXBufData;
	CCyclicBuffer<_SAMPLE>	AudSoDecBuf;

	EAcqStat				eAcquiState;
	int						iAcquDetecCnt;
	int						iGoodSignCnt;
	int						iDelayedTrackModeCnt;
	ERecState				eReceiverState;
	ERecMode				eReceiverMode;
	ERecMode				eNewReceiverMode;

	CSound					SoundInterface;

	_BOOLEAN				bWasFreqAcqu;
	_BOOLEAN				bDoInitRun;

	_REAL					rInitResampleOffset;
};


#endif // !defined(DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_)

==> drm-1.1.tar.gz: drm-1.1/common/DrmReceiver.h <==
/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	See DrmReceiver.cpp
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#if !defined(DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_)
#define DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_

#include <iostream>
#include "Parameter.h"
#include "Buffer.h"
#include "Data.h"
#include "OFDM.h"
#include "DRMSignalIO.h"
#include "MSCMultiplexer.h"
#include "InputResample.h"
#include "datadecoding/DataDecoder.h"
#include "sourcedecoders/AudioSourceDecoder.h"
#include "mlc/MLC.h"
#include "interleaver/SymbolInterleaver.h"
#include "ofdmcellmapping/OFDMCellMapping.h"
#include "chanest/ChannelEstimation.h"
#include "sync/FreqSyncAcq.h"
#include "sync/TimeSync.h"
#include "sync/SyncUsingPil.h"
#include "AMDemodulation.h"
#ifdef _WIN32
# include "../../Windows/source/sound.h"
#else
# include "source/sound.h"
#endif


/* Definitions ****************************************************************/
/* Number of FAC frames until the acquisition is activated in case a signal
   was successfully decoded */
#define	NUM_FAC_FRA_U_ACQ_WITH			12

/* Number of OFDM symbols until the acquisition is activated in case no signal
   could be decoded after previous acquisition try */
#define	NUM_OFDMSYM_U_ACQ_WITHOUT		150

/* Number of FAC blocks for delayed tracking mode switch (caused by time needed
   for initalizing the channel estimation */
#define NUM_FAC_DEL_TRACK_SWITCH		2


/* Classes ********************************************************************/
class CDRMReceiver
{
public:
	/* Acquisition state of receiver */
	enum EAcqStat {AS_NO_SIGNAL, AS_WITH_SIGNAL};

	/* Receiver state */
	enum ERecState {RS_TRACKING, RS_ACQUISITION};

	/* RM: Receiver mode (analog or digital demodulation) */
	enum ERecMode {RM_DRM, RM_AM, RM_NONE};


	CDRMReceiver() : eAcquiState(AS_NO_SIGNAL), iAcquRestartCnt(0),
		iGoodSignCnt(0), bWasFreqAcqu(TRUE), bDoInitRun(FALSE),
		eReceiverMode(RM_DRM), 	eNewReceiverMode(RM_NONE),
		ReceiveData(&SoundInterface), WriteData(&SoundInterface),
		rInitResampleOffset((_REAL) 0.0), iAcquDetecCnt(0)
#ifdef HAVE_LIBHAMLIB
		, strHamlibConf(""), iHamlibModelID(0)
#endif
#ifdef USE_QT_GUI
		, iMainPlotColorStyle(0) /* default color scheme: blue-white */
#endif
		{}
	virtual ~CDRMReceiver() {}

	/* For GUI */
	void					Init();
	void					Start();
	void					Stop();
	EAcqStat				GetReceiverState() {return eAcquiState;}
	ERecMode				GetReceiverMode() {return eReceiverMode;}
	void					SetReceiverMode(ERecMode eNewMode)
								{eNewReceiverMode = eNewMode;}

	void					SetInitResOff(_REAL rNRO)
								{rInitResampleOffset = rNRO;}

	void					SetAMDemodAcq(_REAL rNewNorCen);

	/* Get pointer to internal modules */
	CUtilizeFACData*		GetFAC() {return &UtilizeFACData;}
	CUtilizeSDCData*		GetSDC() {return &UtilizeSDCData;}
	CTimeSync*				GetTimeSync() {return &TimeSync;}
	CChannelEstimation*		GetChanEst() {return &ChannelEstimation;}
	CFACMLCDecoder*			GetFACMLC() {return &FACMLCDecoder;}
	CSDCMLCDecoder*			GetSDCMLC() {return &SDCMLCDecoder;}
	CMSCMLCDecoder*			GetMSCMLC() {return &MSCMLCDecoder;}
	CReceiveData*			GetReceiver() {return &ReceiveData;}
	COFDMDemodulation*		GetOFDMDemod() {return &OFDMDemodulation;}
	CSyncUsingPil*			GetSyncUsPil() {return &SyncUsingPil;}
	CWriteData*				GetWriteData() {return &WriteData;}
	CSound*					GetSoundInterface() {return &SoundInterface;}
	CDataDecoder*			GetDataDecoder() {return &DataDecoder;}
	CAMDemodulation*		GetAMDemod() {return &AMDemodulation;}
	CFreqSyncAcq*			GetFreqSyncAcq() {return &FreqSyncAcq;}


	CParameter*				GetParameters() {return &ReceiverParam;}
	void					StartParameters(CParameter& Param);
	void					SetInStartMode();
	void					SetInTrackingMode();
	void					SetInTrackingModeDelayed();

	void					SetReadDRMFromFile(const string strNFN);

	void					InitsForAllModules();

	void					InitsForWaveMode();
	void					InitsForSpectrumOccup();
	void					InitsForNoDecBitsSDC();
	void					InitsForAudParam();
	void					InitsForDataParam();
	void					InitsForInterlDepth();
	void					InitsForMSCCodSche();
	void					InitsForSDCCodSche();
	void					InitsForMSC();
	void					InitsForMSCDemux();

#ifdef HAVE_LIBHAMLIB
	void					SetHamlibModel(const int iNewM)
								{iHamlibModelID = iNewM;}
	int						GetHamlibModel() const {return iHamlibModelID;}
	void					SetHamlibConf(const string strNewC)
								{strHamlibConf = strNewC;}
	string					GetHamlibConf() const {return strHamlibConf;}
#endif

#ifdef USE_QT_GUI
	/* DRMReceiver object serves as a storage a "exchange platform" for the
	   window size and position parameters for init-file usage. This is not
	   nice but it works for now. TODO: better solution */
	class CWinGeom
	{
	public:
		CWinGeom() : iXPos(0), iYPos(0), iHSize(0), iWSize(0),
			bVisible(FALSE) {}

		int iXPos, iYPos;
		int iHSize, iWSize;
		_BOOLEAN bVisible;
	};

	/* Main window */
	CWinGeom GeomFdrmdialog;

	/* System evaluation window */
	CWinGeom GeomSystemEvalDlg;

	/* Multimedia window */
	CWinGeom GeomMultimediaDlg;

	/* Stations dialog */
	CWinGeom GeomStationsDlg;

	/* Analog demodulation dialog */
	CWinGeom GeomAnalogDemDlg;

	int iMainPlotColorStyle;
#endif

protected:
	void					Run();
	void					DetectAcquiFAC();
	void					DetectAcquiSymbol();
	void					InitReceiverMode();

	/* Modules */
	CReceiveData			ReceiveData;
	CInputResample			InputResample;
	CFreqSyncAcq			FreqSyncAcq;
	CTimeSync				TimeSync;
	COFDMDemodulation		OFDMDemodulation;
	CSyncUsingPil			SyncUsingPil;
	CChannelEstimation		ChannelEstimation;
	COFDMCellDemapping		OFDMCellDemapping;
	CFACMLCDecoder			FACMLCDecoder;
	CUtilizeFACData			UtilizeFACData;
	CSDCMLCDecoder			SDCMLCDecoder;
	CUtilizeSDCData			UtilizeSDCData;
	CSymbDeinterleaver		SymbDeinterleaver;
	CMSCMLCDecoder			MSCMLCDecoder;
	CMSCDemultiplexer		MSCDemultiplexer;
	CAudioSourceDecoder		AudioSourceDecoder;
	CDataDecoder			DataDecoder;
	CWriteData				WriteData;
	CAMDemodulation			AMDemodulation;

	/* Parameters */
	CParameter				ReceiverParam;

	/* Buffers */
	CSingleBuffer<_REAL>	RecDataBuf;
	CCyclicBuffer<_REAL>	FreqSyncAcqBuf;
	CCyclicBuffer<_REAL>	InpResBuf;
	CSingleBuffer<_REAL>	TimeSyncBuf;
	CSingleBuffer<_COMPLEX>	OFDMDemodBuf;
	CSingleBuffer<_COMPLEX>	SyncUsingPilBuf;
	CSingleBuffer<CEquSig>	ChanEstBuf;
	CCyclicBuffer<CEquSig>	MSCCarDemapBuf;
	CCyclicBuffer<CEquSig>	FACCarDemapBuf;
	CCyclicBuffer<CEquSig>	SDCCarDemapBuf;
	CSingleBuffer<CEquSig>	DeintlBuf;
	CSingleBuffer<_BINARY>	FACDecBuf;
	CSingleBuffer<_BINARY>	SDCDecBuf;
	CSingleBuffer<_BINARY>	MSCMLCDecBuf;
	CSingleBuffer<_BINARY>	MSCDeMUXBufAud;
	CSingleBuffer<_BINARY>	MSCDeMUXBufData;
	CCyclicBuffer<_SAMPLE>	AudSoDecBuf;

	EAcqStat				eAcquiState;
	int						iAcquRestartCnt;
	int						iAcquDetecCnt;
	int						iGoodSignCnt;
	int						iDelayedTrackModeCnt;
	ERecState				eReceiverState;
	ERecMode				eReceiverMode;
	ERecMode				eNewReceiverMode;

	CSound					SoundInterface;

	_BOOLEAN				bWasFreqAcqu;
	_BOOLEAN				bDoInitRun;

	_REAL					rInitResampleOffset;

#ifdef HAVE_LIBHAMLIB
	int						iHamlibModelID;
	string					strHamlibConf;
#endif

	int						iSoundCrdDevIn;
	int						iSoundCrdDevOut;
};


#endif // !defined(DRMRECEIVER_H__3B0BA660_CA63_4344_BB2B_23E7A0D31912__INCLUDED_)
